from lionweb.model import Node
from lionweb.language import IKeyed, Language

def lw_id_cleaned_version(s: str) -> str:
    return s.replace(".", "_").replace(" ", "_").replace("/", "_")


def id_prefix_for_contained_elements(node: Node) -> str:
    return node.id.removeprefix("language-").removesuffix("-id")

def id_for_contained_element(node: Node, contained_element_name: str) -> str:
    return f"{id_prefix_for_contained_elements(node)}-{lw_id_cleaned_version(contained_element_name)}-id"

def default_id(node: Node) -> str:
    curr = node
    while curr is not None and not isinstance(curr, Language):
        curr = curr.get_parent()
    suffix = ""
    if curr is not None and curr.get_version() != '1':
        suffix = f"-{curr.get_version()}"
    return f"{id_prefix_for_contained_elements(node.get_parent())}-{lw_id_cleaned_version(node.get_name())}{suffix}-id"

def key_prefix_for_contained_elements(keyed: IKeyed) -> str:
    return keyed.key.removeprefix("language-").removesuffix("-key")

def key_for_contained_element(keyed: IKeyed, contained_element_name: str) -> str:
    return f"{key_prefix_for_contained_elements(keyed)}-{lw_id_cleaned_version(contained_element_name)}-key"

def default_key(node: Node) -> str:
    return f"{key_prefix_for_contained_elements(node.get_parent())}-{lw_id_cleaned_version(node.get_name())}-key"
